// License: GPLv3
// by Leszek Lesner
/////////////////////
// Dependency: SSH_ASKPASS client like ksshaskpass


#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->connectBtn->setDefault(true);
    ui->connectBtn->setEnabled(false);
    ui->serverEdit->setFocus();
    center();
    genMenu();

    /*
     * Connect Signals to Slots here
     *
     */
    // SFTP
    connect(connectRun, SIGNAL( finished ( int, QProcess::ExitStatus)), this, SLOT(checkStatus(int,QProcess::ExitStatus)));
    connect(connectRun, SIGNAL(readyReadStandardError()), this, SLOT(updateError()));
    connect(connectRun, SIGNAL(readyReadStandardOutput()), this, SLOT(updateOutput()));
    // Samba/CIFS
    connect(sambaShares, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(checkShares(int,QProcess::ExitStatus)));
    connect(sambaShares, SIGNAL(readyReadStandardOutput()), this, SLOT(updateShares()));
    connect(sambaShares, SIGNAL(readyReadStandardError()), this, SLOT(updateError()));

}

MainWindow::~MainWindow()
{
    delete ui;
}

/*
 * Center the window
 *
 */
void MainWindow::center() {
    QDesktopWidget *desktop = QApplication::desktop();

    int screenWidth, width;
    int screenHeight, height;
    int x, y;
    QSize windowSize;

    screenWidth = desktop->width(); // get width of screen
    screenHeight = desktop->height(); // get height of screen

    windowSize = size(); // size of our application window
    width = windowSize.width();
    height = windowSize.height();

    // little computations
    x = (screenWidth - width) / 2;
    y = (screenHeight - height) / 2;
    y -= 50;

    // move window to desired coordinates
    move ( x, y );

    if (ui->typeBox->currentText() == "SFTP") {
        ui->label_3->hide();
        ui->password->hide();
    }
}

/*
 * Generate configure menu here
 *
 */
void MainWindow::genMenu() {
    QMenu *menu = new QMenu();
    QAction *aboutAction = new QAction("About", this);
    QAction *unmountAction = new QAction("Unmount", this);
    menu->addAction(unmountAction);
    menu->addSeparator();
    menu->addAction(aboutAction);
    ui->menuButton->setMenu(menu);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(showAbout()));
    connect(unmountAction, SIGNAL(triggered()), this, SLOT(unmount()));
}

/*
 * About Window
 *
 */
void MainWindow::showAbout() {
    QMessageBox about;
    about.setText("<b>Knetworkmounter 0.6</b>");
    about.setInformativeText("by Leszek Lesner\nreleased under the terms of GPLv3");
    QPixmap *icon;
    about.setIconPixmap(icon->fromImage(QImage(":/knetworkmanager_icon.png")));
    about.setWindowTitle("About");
    about.exec();
}

/*
 * Open unmounter
 *
 */
void MainWindow::unmount() {
    QFile unmountScript(":/unmount.sh");
    if (QFile("/tmp/unmount.sh").exists()) {
        QFile("/tmp/unmount.sh").remove();
    }
    unmountScript.copy("/tmp/unmount.sh");
    system("bash /tmp/unmount.sh");
}

/*
 * Enable connect button only if server edit textfield has text in it
 *
 */
void MainWindow::on_serverEdit_cursorPositionChanged()
{
    if (MainWindow::ui->serverEdit->text().length() != 0) {
        // qDebug() << "Debug: serverEdit length = " + QString::number(MainWindow::ui->serverEdit->text().length()); // DEBUG
        MainWindow::ui->connectBtn->setEnabled(true);
    } else {
        MainWindow::ui->connectBtn->setEnabled(false);
    }
}

void MainWindow::on_connectBtn_clicked()
{
    if (ui->typeBox->currentText() == "SFTP") {
        ui->connectBtn->setEnabled(false);
        ui->connectBtn->setText(tr("Connecting..."));
        ui->statusBar->showMessage(tr("Connecting to ") + ui->serverEdit->text() ,0);
        lastIP = ui->serverEdit->text();
        makeSFTPConnection(ui->serverEdit->text(),ui->username->text(),ui->password->text());
    }
    else if (ui->typeBox->currentText() == "Samba/CIFS") {
        ui->connectBtn->setEnabled(false);
        ui->connectBtn->setText(tr("Connecting..."));
        ui->statusBar->showMessage(tr("Connecting to ") + ui->serverEdit->text() ,0);
        lastIP = ui->serverEdit->text();
        makeSAMBAConnection(ui->serverEdit->text(),ui->username->text(),ui->password->text());
    }
    else ui->statusBar->showMessage(tr("Not implemented yet!"),10000);
}

void MainWindow::makeSFTPConnection(QString ip, QString user, QString pw)
{
    system("mkdir -p /tmp/mnt/" + ip.toLatin1());
    //QString connectCmd = "/bin/sh -c \" echo "+ pw + " | sshfs -o reconnect -o password_stdin " + user + "@" + ip +":/ /tmp/mnt/"+ip + " \"";
    // We now use sshfs directly and have ksshaskpass as dependency or any other graphical SSH_ASKPASS client
    QString connectCmd = "sshfs -o reconnect " + user + "@" + ip +":/ /tmp/mnt/"+ip + "";
    qDebug() << "Command is:" + connectCmd;
    //    QProcess *connectRun = new QProcess();  // Create new Process
    lastMountpoint = "/tmp/mnt/" +ip;
    connectRun->start(connectCmd); // start process
    //    system(connectCmd.toLatin1());
}

void MainWindow::makeSAMBAConnection(QString ip, QString user, QString pw)
{
    // 1. Ask the Server for Samba shares
    QString sambaSharesCmd;
    if ((user.length() == 0) && (pw.length()==0))
    {
        sambaSharesCmd = "/bin/sh -c \"smbclient -N -L //" + ip + " 2> /dev/null|grep -w Disk|cut -d ' ' -f 1\" ";
    }
    else {
        sambaSharesCmd = "/bin/sh -c \"smbclient -U " + user +"[" + pw + "] -N -L //" + ip + " 2> /dev/null|grep -w Disk|cut -d ' ' -f 1\" ";
    }
    sambaShares->start(sambaSharesCmd);
    sD->smbIp = ip;
    sD->smbUser = user;
    sD->smbPw = pw;
}

/*
 * Error checking for samba shares
 *
 */
void MainWindow::checkShares(int stat,QProcess::ExitStatus)
{
    if (stat != 0) {
        qDebug() << "Connection failed: " + QString::number(stat);
        qDebug() << "Error message: " + output;
        ui->statusBar->showMessage(tr("Connection failed or no Samba shares available!"),10000);
    } else {
        sD->show();
        setConnectStatus("finished");
    }
}

/*
 * Error checking for mounts
 *
 */
void MainWindow::checkStatus(int stat,QProcess::ExitStatus)
{
    if (stat != 0) {
        qDebug() << "Connection failed: " + QString::number(stat);
        qDebug() << "Error message: " + output;
        ui->statusBar->showMessage(tr("Connection failed!"),10000);
        // guess it is already mounted
        if (output.contains("mountpoint is not empty")) {
            setConnectStatus("finished");
            QProcess *filemanagerStart = new QProcess(this);
            filemanagerStart->start("dolphin " + lastMountpoint.toLatin1());
        }
        ui->connectBtn->setEnabled(true);
    } else {
        qDebug() << "Connection success: "+ QString::number(stat);
        qDebug() << "Output message: " + output;
        // Open filemanager if connection succeeded
        setConnectStatus("finished");
        QProcess *filemanagerStart = new QProcess(this);
        filemanagerStart->start("dolphin " + lastMountpoint.toLatin1());
    }
}

void MainWindow::setConnectStatus(QString status) {
    if (status == "finished") {
        ui->connectBtn->setText(tr("Connect"));
        ui->connectBtn->setEnabled(true);
        ui->serverEdit->clear();
        ui->username->clear();
        ui->password->clear();
        ui->statusBar->showMessage(tr("Connected to") + lastIP ,10000);
    }
    else {
        ui->connectBtn->setEnabled(false);
        ui->connectBtn->setText(tr("Connecting"));
        ui->statusBar->clearMessage();
    }
}

/*
 * Update error log
 *
 */
void MainWindow::updateError()
{
    QByteArray data = connectRun->readAllStandardError();
    output = QString(data);
}

/*
 * Update output log
 *
 */
void MainWindow::updateOutput()
{
    QByteArray data = connectRun->readAllStandardOutput();
    output = QString(data);
}

/*
 * Update shares log
 *
 */
void MainWindow::updateShares()
{
    while (sambaShares->canReadLine()) {
        QString line = sambaShares->readLine();
        sD->addShare(line.trimmed());
    }
}


void MainWindow::on_abortBtn_clicked()
{
    exit(0);
}

void MainWindow::on_serverEdit_returnPressed()
{
    on_connectBtn_clicked();
}

void MainWindow::on_password_returnPressed()
{
    on_connectBtn_clicked();
}

void MainWindow::on_typeBox_currentIndexChanged(const QString &arg1)
{
    if (arg1 == "SFTP") {
        ui->label_3->hide();
        ui->password->hide();
    }
    else {
        ui->label_3->show();
        ui->password->show();
    }
}

void MainWindow::on_username_returnPressed()
{
    on_connectBtn_clicked();
}
