#include "sharesdialog.h"
#include "ui_sharesdialog.h"

sharesDialog::sharesDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::sharesDialog)
{
    ui->setupUi(this);
    connect(smbConnectRun, SIGNAL( finished ( int, QProcess::ExitStatus)), this, SLOT(checkStatus(int,QProcess::ExitStatus)));
    connect(smbConnectRun, SIGNAL(readyReadStandardError()), this, SLOT(updateError()));
    connect(smbConnectRun, SIGNAL(readyReadStandardOutput()), this, SLOT(updateOutput()));
}

sharesDialog::~sharesDialog()
{
    delete ui;
}

void sharesDialog::addShare(QString share)
{
    ui->listWidget->addItem(share);
}

void sharesDialog::on_buttonBox_rejected()
{
    ui->listWidget->clear();
    this->close();
}

void sharesDialog::on_buttonBox_accepted()
{
    QString smbFolder = ui->listWidget->currentItem()->text();
    system("mkdir -p /tmp/mnt/" + smbIp.toLatin1() + "-" + smbFolder.toLatin1());
    lastMountpoint = "/tmp/mnt/" +smbIp + "-" + smbFolder;
    // Login as guest
    QString connectCmd;
    if ((smbUser.length() == 0) && (smbPw.length() == 0)) {
        connectCmd = "/usr/bin/kdesudo -d --noignorebutton \"mount -t cifs -o password=nothing //"+ smbIp + "/"+ smbFolder +" /tmp/mnt/" + smbIp + "-" + smbFolder + "\"";
    }
    else {
        connectCmd = "/usr/bin/kdesudo -d --noignorebutton \"mount -t cifs -o username=" + smbUser + ",password=" + smbPw + " //"+ smbIp + "/"+ smbFolder +" /tmp/mnt/" + smbIp + "-" + smbFolder + "\"";
    }
    qDebug() << "Command is:" + connectCmd;
    smbConnectRun->start(connectCmd); // start process
    ui->listWidget->clear();
}

/*
 * Error checking for mounts
 *
 */
void sharesDialog::checkStatus(int stat,QProcess::ExitStatus)
{
    if (stat != 0) {
        qDebug() << "Connection failed: " + QString::number(stat);
        qDebug() << "Error message:" + output;
        QMessageBox::warning(0,tr("Connection error"),tr("Connection failed!"),QMessageBox::Ok);
        // guess it is already mounted
        if (output.contains("Device or resource busy")) {
            QProcess *filemanagerStart = new QProcess();
            filemanagerStart->start("dolphin " + lastMountpoint.toLatin1());
        }
    } else {
        qDebug() << "Connection success: "+ QString::number(stat);
        // Open filemanager if connection succeeded
        QProcess *filemanagerStart = new QProcess();
        filemanagerStart->start("dolphin " + lastMountpoint.toLatin1());
    }
}

/*
 * Update output log
 *
 */
void sharesDialog::updateOutput()
{
    QByteArray data = smbConnectRun->readAllStandardOutput();
    output = QString(data);
}

/*
 * Update error log
 *
 */
void sharesDialog::updateError()
{
    QByteArray data = smbConnectRun->readAllStandardError();
    output = QString(data);
}
